/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <gnome.h>
#include <gtk/gtk.h>
#include <pthread.h>
#include "pan.h"

void server_free(server_data * sdata)
{
	if (sdata) {
		g_free(sdata->address);
		g_free(sdata->port);
		g_free(sdata);
	}
}

static server_data *server_fetch(char *name)
{
	server_data *sdata = NULL;
	char buf[256];

	g_snprintf(buf, 256, "/Pan/%s/", name);
	gnome_config_push_prefix(buf);

	if (gnome_config_has_section(buf)) {
		sdata = (server_data *) g_malloc0(sizeof(server_data));
		if ((sdata->address = gnome_config_get_string("Address")) == NULL) {
			g_free(sdata);
			sdata = NULL;
		} else {
			sdata->port =
				gnome_config_get_string_with_default("Port=119", NULL);
			sdata->name = g_strdup(name);
			sdata->username = gnome_config_get_string("Username");
			sdata->password = gnome_config_get_string("Password");
		}
	}
	gnome_config_pop_prefix();

	return sdata;
}

void server_set_active(void)
{
	char path[PATH_MAX];
	char *name;	
	
	name = gnome_config_get_string("/Pan/Servers/Active");

	if (name) {
		stop_all_online_tasks();
		/* We lock the mutex to insure that we are really
		 * offline since stop_all_online_tasks can take
		 * awhile to achieve it's goal in the queue thread */
		g_snprintf (path, PATH_MAX, "/%s/%s/", props_get_data_dir(), name);
		directory_check (path);

		server_free (Pan.active_server);
		Pan.active_server = server_fetch (name);
		article_ctree_clean ();
		grouplist_free (Pan.grouplist);
		Pan.grouplist = NULL;
		grouplist_update ();
		g_free (name);
	}
}

void server_init(void)
{

	/* This gets called before gtk_main and
	 * at the end of guru_finish */

	server_set_active();
	server_menu_update();
}

void
server_selected_cb (GtkWidget *menuitem, char *name)
{
	g_return_if_fail(name);

	if (is_online()) {
		gnome_ok_dialog(_("The active server cannot be changed while online."));
		return;
	}
	
	if (Pan.active_server && !strcmp(name, Pan.active_server->name)) {
		return;
	} else {
		gnome_config_set_string("/Pan/Servers/Active", name);
		gnome_config_sync();
		server_set_active();
		server_menu_update();
    }
}

void
server_menu_update (void)
{
    int i;
	char *p;
	static char **sd = NULL;
	static int server_items = 0;
	GnomeUIInfo entry[2];

	gnome_app_remove_menus(GNOME_APP(Pan.window), "Servers/", server_items);
	server_items = 0;
	g_strfreev(sd);

    if ((p = gnome_config_get_string("/Pan/Servers/Names")) == NULL)
        return;
    
	sd = g_strsplit(p, ",", 0);
	g_free(p);
	
	g_return_if_fail(sd);

    p = gnome_config_get_string("/Pan/Servers/Active");
    for (i = 0; sd[i]; i++) {
		entry[0].type = GNOME_APP_UI_ITEM;
		entry[0].label = sd[i];
		entry[0].hint = NULL;
		entry[0].moreinfo = server_selected_cb;
		entry[0].user_data = sd[i];
		entry[0].unused_data = NULL;
		if (p && !strcmp(p, sd[i])) {
			entry[0].pixmap_type = GNOME_APP_PIXMAP_STOCK;
			entry[0].pixmap_info = GNOME_STOCK_PIXMAP_CLOSE;
		} else {
			entry[0].pixmap_type = GNOME_APP_PIXMAP_NONE;
			entry[0].pixmap_info = NULL;
		}
		entry[0].accelerator_key = 0;
		entry[1].type = GNOME_APP_UI_ENDOFINFO;
		gnome_app_insert_menus(GNOME_APP(Pan.window), "Servers/", entry);
		server_items ++;
	}
	g_free(p);	
	/*
	if (menu_sensitized == FALSE) {
   		server_menu_desensitize();
    }
	*/
}


